@@ -10,15 +10,24 @@ KUBE_USER = "zalando-token"
 
 def update(url, alias, token):
     name = alias or generate_name(url)
+    config = read_config()
+
+    username = KUBE_USER
+    if alias:
+        okta_user = "okta-{}".format(alias)
+        # To handle weird things like "users: null"
+        users = (config or {}).get("users") or []
+        if any(user.get("name", "") == okta_user for user in users):
+            username = okta_user
+
     new_config = {
         "apiVersion": "v1",
         "kind": "Config",
         "clusters": [{"name": name, "cluster": {"server": url}}],
         "users": [{"name": KUBE_USER, "user": {"token": token}}],
-        "contexts": [{"name": name, "context": {"cluster": name, "user": KUBE_USER}}],
+        "contexts": [{"name": name, "context": {"cluster": name, "user": username}}],
         "current-context": name,
     }
-    config = read_config()
     updated_config = merge_config(config, new_config)
     if updated_config != config:
         write_config(updated_config)
