@@ -21,6 +21,7 @@ import zalando_kubectl
 import zalando_kubectl.kube_permissions
 import zalando_kubectl.secrets
 import zalando_kubectl.traffic
+import zalando_kubectl.utils as utils
 from clickclick import Action, info, print_table
 from dateutil.parser import parse
 from dateutil.tz import tzutc
@@ -33,7 +34,6 @@ from zalando_kubectl.utils import (
     auth_headers,
     auth_token,
     current_user,
-    get_api_server_url,
 )
 
 from . import access_request
@@ -65,6 +65,12 @@ def global_options(fn):
         if not ctx.obj:
             ctx.obj = Environment()
         if value is not None:
+            # kubectl allows -s=bar in addition to --long=bar, which Click parses
+            # as -s "=bar". Since neither of these can start with a '=' anyway,
+            # let's just trim it.
+            if value.startswith("="):
+                value = value[1:]
+
             if param.name == "namespace":
                 ctx.obj.set_namespace(value)
             elif param.name == "context":
@@ -169,7 +175,6 @@ def login(config, cluster_or_url: str):
         alias = cluster["alias"]
         configure_zdeploy(cluster)
 
-    config["api_server"] = url
     return url, alias
 
 
@@ -332,7 +337,7 @@ def encrypt(env: Environment, cluster, kms_keyid, strip):
     if cluster:
         cluster_metadata = registry.get_cluster_by_id_or_alias(registry_url, cluster)
     else:
-        cluster_metadata = registry.get_cluster_with_params(registry_url, api_server_url=get_api_server_url(env.config))
+        cluster_metadata = registry.get_cluster_with_params(registry_url, api_server_url=utils.get_api_server_url(env))
 
     if not kms_keyid:
         local_id = cluster_metadata["id"].split(":")[-1]
@@ -457,7 +462,7 @@ def cluster_update():
 def cluster_update_status(env: Environment):
     """Show the cluster update status"""
     registry_url = get_registry_url(env.config)
-    api_server_url = get_api_server_url(env.config)
+    api_server_url = utils.get_api_server_url(env)
     cluster_metadata = registry.get_cluster_with_params(registry_url, verbose=True, api_server_url=api_server_url)
     alias = cluster_metadata["alias"]
 
@@ -480,7 +485,7 @@ def cluster_update_status(env: Environment):
 def block_cluster_update(env: Environment):
     """Block the cluster from updating"""
     registry_url = get_registry_url(env.config)
-    api_server_url = get_api_server_url(env.config)
+    api_server_url = utils.get_api_server_url(env)
     cluster_metadata = registry.get_cluster_with_params(registry_url, verbose=True, api_server_url=api_server_url)
     alias = cluster_metadata["alias"]
 
@@ -501,7 +506,7 @@ def block_cluster_update(env: Environment):
 def unblock_cluster_update(env: Environment):
     """Allow updating the cluster"""
     registry_url = get_registry_url(env.config)
-    api_server_url = get_api_server_url(env.config)
+    api_server_url = utils.get_api_server_url(env)
     cluster_metadata = registry.get_cluster_with_params(registry_url, verbose=True, api_server_url=api_server_url)
     alias = cluster_metadata["alias"]
 
@@ -799,7 +804,7 @@ def cluster_access():
 def cluster_access_check_environment(env: Environment):
     try:
         registry_url = get_registry_url(env.config)
-        cluster_metadata = registry.get_cluster_with_params(registry_url, api_server_url=get_api_server_url(env.config))
+        cluster_metadata = registry.get_cluster_with_params(registry_url, api_server_url=utils.get_api_server_url(env))
         if cluster_metadata["environment"] == "test":
             raise ClusterAccessUnsupported("Cluster access requests are not needed in test clusters")
     except ClusterAccessUnsupported:
@@ -811,7 +816,7 @@ def cluster_access_check_environment(env: Environment):
 
 @cluster_access.command("request")
 @click.option("--emergency", is_flag=True, help="Request emergency access to the cluster")
-@click.option("-i", "--incident", help="Incident number, required with --emergency", type=str)
+@click.option("-i", "--incident", help="Opsgenie incident number or URL, required with --emergency", type=str)
 @click.option("-u", "--user", help="User that should be given access, defaults to the current user", required=False)
 @click.argument("reason", nargs=-1, required=True)
 @click.pass_obj
@@ -822,28 +827,36 @@ def request_cluster_access(env: Environment, emergency, incident, user, reason):
     if emergency:
         if not incident:
             raise click.UsageError("You must specify an incident ticket [--incident] when requesting emergency access")
-        if incident.startswith("INC-"):
-            incident = re.sub(r"INC-", "", incident)
-        if not incident.isdigit():
-            raise click.UsageError("You must specify a valid incident ticket, e.g. INC-1234 or 1234")
-        make_emergency_access_request(env.config, incident, user, reason)
+        if incident.startswith("https://"):
+            reference_url = incident
+        elif incident.startswith("INC-"):
+            reference_url = "https://jira.zalando.net/browse/" + incident
+        elif incident.isdigit():
+            # This way we can be sort of compatible with both old and new incident processes
+            reference_url = "https://jira.zalando.net/browse/INC-" + incident
+        elif re.fullmatch(r"\w+-\w+-\w+-\w+-\w+", incident):
+            reference_url = "https://zalando.app.opsgenie.com/incident/detail/" + incident
+        elif re.fullmatch(r"\w+-\w+-\w+-\w+-\w+-\d+", incident):
+            reference_url = "https://zalando.app.opsgenie.com/alert/show/" + incident
+        else:
+            raise click.UsageError("You must provide a valid incident number or ID, alert ID or an Opsgenie URL")
+        make_emergency_access_request(env, reference_url, user, reason)
     else:
-        make_manual_access_request(env.config, user, reason)
+        make_manual_access_request(env, user, reason)
 
 
-def make_emergency_access_request(config, incident, user, reason):
+def make_emergency_access_request(env: Environment, reference_url, user, reason):
     user = access_request_username(user)
-    reference_url = "https://jira.zalando.net/browse/INC-{}".format(incident) if incident else None
-    access_request.create(config, "emergency", reference_url, user, " ".join(reason))
+    access_request.create(env, "emergency", reference_url, user, " ".join(reason))
     click.echo(
         "Emergency access provisioned for {}. Note that it might take a while "
         "before the new permissions are applied.".format(user)
     )
 
 
-def make_manual_access_request(config, user_option, reason):
+def make_manual_access_request(env: Environment, user_option, reason):
     user = access_request_username(user_option)
-    access_request.create(config, "manual", None, user, " ".join(reason))
+    access_request.create(env, "manual", None, user, " ".join(reason))
     click.echo("Requested manual access for {}.".format(user))
     click.echo("You can ask your colleague to approve it using these commands:\n")
     current_cluster = kube_config.get_current_context()
@@ -863,7 +876,7 @@ def approve_access_request(env: Environment, username):
     if not username:
         username = click.prompt("User that should receive access")
 
-    existing = access_request.get_all(env.config).get(username)
+    existing = access_request.get_all(env).get(username)
     if not existing:
         clickclick.warning("No access requests for {user}".format(user=username))
     elif existing["approved"]:
@@ -871,7 +884,7 @@ def approve_access_request(env: Environment, username):
     else:
         reason = existing.get("reason", "(no reason provided)")
         if click.confirm("Manual access request from {user}: {reason}. Approve?".format(user=username, reason=reason)):
-            updated_reason = access_request.approve(env.config, username)
+            updated_reason = access_request.approve(env, username)
             click.echo("Approved access for user {user}: {reason}".format(user=username, reason=updated_reason))
 
 
@@ -881,7 +894,7 @@ def list_cluster_requests(env: Environment):
     """List all current pending/approved access requests for the cluster"""
     cluster_access_check_environment(env)
 
-    all_requests = access_request.get_all(env.config)
+    all_requests = access_request.get_all(env)
     access_requests = sorted(all_requests.values(), key=lambda r: r["user"])
 
     rows = []
